#ifndef __CONTR_C
#define __CONTR_C

/* Set the contrast on a TI-83

   Small-C library, by Henk Poley (hpoley@dds.nl)
   (---no www addres---)

   Synopsis:
      getcont()
      setcont(int cont)
      cont_up()
      cont_down()

   Description:
      o getcont()
        Returns the contrast. (what the TIOS (8008h) says of it)

      o setcont(int cont)
       - Change the contrast to cont. And returns the (new) contrast.

       - cont = 0 to 31

       - Increasing cont with 4 is the same as pressing "2nd + kUp"
         till the next number appears.

      o cont_up()
        Contrast increased by 1 (not the number you see on you calc)
        Returns the (new) contrast.

      o cont_down()
        Contrast decreased by 1 (not the number...)
        Returns the (new) contrast.

   Example:
      contr = getcont()
      setcont(10)  ; I think this is contrast '4'
      cont_up()
      cont_down()

   Output:
      contrast changed (whoww....). And you will get the contrast
      number stored by TIOS. (Of coarse, getcont() will only return
      the contrast number and won't set it...)

   Notes:
      o Now gigantic optimized (5-9-2000)

      o !!! NOTE !!!
        The Ti-82 ROM uses number from 0 - 31, and the Ti-83 0 - 39...
        So these routines act different... (Ti-85 also uses 0 - 31!)
        BTW. the LCD-driver-chip is the same as the one on the Ti-83.
        So it is possible to set an higher contrast level. I don't know
        if the calc might become unstable, so I don't use it.

      o If somebody knows where the contrast changing routine is in the
        ROM, so you would get these 'funny' numbers, and you think it
        needs it..., please adapt these routines.
        I know you could calculate the 'funny' number yourself and
        display it in inverted text... But that would make things
        way too big.

      o I 'stole' this routine a bit from Andrew Von Dollen, but he
        doesn't seem to worry about this. I found his routine in
        AsmGuru. I changed a lot to make it work with the Small-C
        compiler.

      o There are a few things I like to have added/changed:
        - If I don't use "call _lcd_busy" everything still works fine
          on my VirtualTi. The called routine is only a delay between
          writes. And I haven't written to the LCD for a while...
          So I don't think it is really needed, correct me if I'm
          wrong.

        - All the routines return the (new) value of the contrast.
          Is this usefull?
        
      o I found a smaller 'routine' for _lcd_busy, in FlagScan by Jason
        Kovacs (Jason_K@calc.org). Use his 'routine' if necesairy.

	;
	EX	(sp),hl
	EX	(sp),hl
	;
	; These two commands together are used instead of calling
	; an LCD_Busy routine, which wastes bytes. Together, these
	; commands cancel eachother out and dont mess up the stack
	; yet they still provide a sufficient enough Delay before
	; each Out Instruction, with 38 T-States on Each Pass.

	_lcd_busy on the Ti-82 is at 07F3h in ROM


      o Some doc. I found on contrast changing on the Ti-82:

                     Set contrast
                     ------------
Any command writen to port 10 which has the 2 MSB set will alter the
contrast. The valid values for the contast is 00 to 3F, with 3F being
the darkest and 00 the lightest. So to set the contrast to the value in 
a use the following code (a has to hold a valid contrast value).

    or   0C0h         ; Set the two MSB's
    call 7F3h         ; Delay use for display commands
    out  (10h),a      ; Write command to port
    ret

Note that this function will set the contrast, but it will be returned
to the previus value if the build in functions are used. The system
functions uses contrast values 0 to 20, and the current value is stored
in the system part of the RAM. To avoid that the contrast is set to
what it was before remember to change the value in the system RAM too.
*/

#asm
#DEFINE	curcont		8008h	; Current contrast level (0..1F)
#endasm

getcont() {
#asm
	ld	hl,(curcont)	; Whooww, this is a some BIG code :)
#endasm
}

setcont(cont) int cont;
{
#asm
	ld	hl,2		; 2 is the offset of the last parameter
	add	hl,sp		; Add the stack to the offset
	ld	a,(hl)		; a  = (hl)
	ld	hl,(curcont)	; hl = current contrast
	cp	31		; If(a > 31)
	ret	nc		;  return(curcont);
	ld	l,a		; hl = curent contrast
	ld	(curcont),a	; Save new contrast for TIOS
	or	0C0h		; Set the two MSB's
	out	(11),a		; Change the contrast
				; return(curcont)
#endasm
}

cont_up()
{
#asm
	ld	hl,(curcont)	; hl = current contrast
	ld	a,l		; a  = current contrast
	cp	31		; If(a = 31)
	ret	z		;  return(curcont);
	inc	a		; a++
	ld	(curcont),a	; Save new contrast for TIOS
	or	0C0h		; Set the two MSB's
	out	(11),a		; Engage!
	inc	l		; return(curcont)
#endasm
}

cont_down()
{
#asm
	ld	hl,(curcont)	; hl = current contrast
	ld	a,l		; a  = current contrast
	dec	a		; a--
	ret	m		; if(a = 'negative')
				;  return(curcont);
	ld	(curcont),a	; Save new contrast for TIOS
	or	0C0h		; Set the two MSB's
	out	(11),a		; Make it so!
	dec	l		; return(curcont)
#endasm
}
#endif