#ifndef MALLOC_C
#define MALLOC_C

/* Small c library, by Jeremy Dewey (jxd45@po.cwru.edu)
   http://www.seashellinc.com/jdewey

   Synopsis:
      init_mem()
      malloc(int size)
      free(char* ptr)

   Description:
      init_mem() initializes the memory manager, returning the amount of free
      heap memory.
      malloc() allocates 'size' bytes of memory, returning a pointer to it
      free() deallocates the memory pointed to by 'ptr'

   Example: 
      init_mem();
      ptr = malloc(32);
      free(ptr);

   Notes:
      A perfectly competent implementation of memory management with a very
      simple design. Memory is allocated in even-sized blocks, each block
      containing a 2-byte header. The lower 15 bits of the header give the block
      size, and the 16th bit is set if memory is in use.

      In order to use malloc() and free(), you must first call init_mem().

      WARNING: This implementation has one fault -- if you write any data before
      or after the area allocated it is likely to blow. Not too forgiving or
      efficient, but extremely simple.
*/

extern char* _lastmem(0x8b25);
extern char* _frstmem(0x8b27);

/* Returns the amount of free memory */

init_mem() {
   int* mem;
   mem = _frstmem;
   mem[0] = _lastmem - _frstmem - 3;
   return mem[0];
}

/* Returns a pointer to free memory or 0 if none available */
malloc(size) int size; {
   int* mem;
   int oldsize;
   mem = _frstmem;

   if (size & 0x0001) size++;
   while (*mem & 0x8000 || *mem < size+2) {
      mem = heapwalk(mem);
      /* Couldn't find any free memory */
      if (mem >= _lastmem - 3) return 0;
   }
   /* Found some space */
   oldsize = *mem;
   *mem = size | 0x8000;
   if (oldsize > size) *(heapwalk(mem)) = oldsize - size - 2;
   return mem + 1;
}

/* Frees block and merges with neighbors if possible */
free(ptr) int* ptr; {
   int *mem, *prev, *next;
   if (!ptr) return;

   /* Make ptr point to block header */
   ptr--;
   *ptr = *ptr & 0x7fff;
   prev = mem = _frstmem;
   while (mem < ptr) {
      prev = mem;
      mem = heapwalk(mem);
   }
   /* mem should now equal ptr, prev should be the block preceeding ptr */
   /* Merge with next block if able */
   if (mem < _lastmem - 3) {
      next = heapwalk(mem);
      if (!(*next & 0x8000))
         *mem = *mem + *next + 2;
   }
   /* Merge with previous block if able */
   if (mem != prev && !(*prev & 0x8000))
      *prev = *prev + *mem + 2;
}

/* Given a pointer to a block header, returns the next block header */
heapwalk(ptr) int* ptr; {
   /* Annoying pointer arithmetic... */
   return ptr + ((*ptr & 0x7fff) >> 1) +1;
}

