/*
 *      dstar.c
 *
 *      dstarz88 is a conversion of a TI86 game I found with 
 *      source on www.ticalc.org.
 *
 *      The original program was written by Andrew Von Dollen who
 *      in turn based it on a HP game by Joe W.
 *
 *      The aim of the game is to collect all the clear bubbles by
 *      running over them. You control either the dark bubble or
 *      the solid box. The dark bubble is used to collect the clear
 *      bubbles, and the solid box is used as a sort of movable wall.
 *
 *      Both objects carry on moving until they hit something else
 *      (except for the dark bubble in the case of clear bubbles).
 *
 *      The keys are defined in #define statements, and default thus:
 *
 *      Up:     Q
 *      Down:   A
 *      Left:   O
 *      Right:  P
 *      Quit:   G
 *      Retry:  H
 *      Switch: [SPACE]
 *
 *      Switch changes between the dark bubble and the solid box.
 *
 *      This is the first game ever produced with the Small C compiler -
 *      it was written as a statement saying that it is possible to
 *      write something easily, quickly and efficiently using the
 *      compiler. Hopefully it will be an encouragement for others to
 *      do likewise!
 *
 *      For your interest I've also included the original Z88 converted
 *      assembler from which I worked - it's quite crude but it just
 *      about functions, the C is much more refined!
 *
 *      Enough twaddle, enjoy the game and study the source!
 *
 *      d. <djmjb.man.ac.uk> 1/12/98
 *
 *
 *      Rejigged for the Spectrum 17/5/99 djm
 *
 */

#include <ti.h>
#include <fastcopy.h>
#include <exit.c>

#define NO 0
#define MAXLEVEL 25
#define STARTLEV  0     /* Start level -1 */

/* Block numbers.. */

#define WALL 1  
#define BUBB 2
#define BALL 3
#define BOX 4

/* Key definitions, change these to define your keys! */
/* Ti8xcc defines them by default...
#define kUP     'Q'
#define kDOWN   'A'
#define kLEFT   'O'
#define kRIGHT  'P'
#define kEXIT   'G'
#define kCLEAR  'H'
*/
#define kSWITCH kMATH	/* kSWITCH is not a regular key */


/* Declare some variables to start off with! */
char balloffset;        /* Ball position */
char boxoffset;         /* Box position */
char ballorbox;         /* 1 if box, 0 if ball */
char level;             /* Take a guess! */

char board[144];        /* Level internal map thing */
char tiscr[1024];       /* Our very own TI86 screen! */

extern char levels[];
extern char sprites[];

main()
{
        rdrwscr();         /* Define the windows */
        playgame();     /* Play the game */
}

playgame()
{
        setupgame();            /* Set level to 1, get level etc */
/* Loop while checkfinish() says we haven't finished! */

        while ( checkfinish() ) {
                gamekeys();     /* Scan keys */
        }
}


/* Set some variables up at the start.. */

setupgame()
{
        ballorbox = NO;
        level     = STARTLEV;
        setuplevel();
}


gamekeys()
{
        char *charptr;

/* Set up a pointer to the variable we want to change (either for
 * box or for ball
 */
        if (ballorbox) charptr = &boxoffset;
        else           charptr = &balloffset;

        switch(getkey()) {
                case kDown:
                        down(charptr);
                        break;
                case kUp:
                        up(charptr);
                        break;
                case kRight:
                        right(charptr);
                        break;
                case kLeft:
                        left(charptr);
                        break;
                case kSWITCH:
                        ballorbox^=1;   /* Toggle ball/box */
                        break;
                case kExit:
                        exit(0);
                case kClear:
                        setuplevel();
        }
}


/* Movement functions - all of these are pretty well similar so I
 * will only comment the first one - it's fairly obvious what is
 * happening though
 */

left(ptr) char *ptr;
{
        char *locn;

        while(1) {
                locn      = *(ptr) + board;
                if (standardmiddle(*(locn - 1))) return;
                *(locn-1) = *locn;
                *locn     = 0;
                (*ptr)--;       /* ptr is the location of blob */
                drawboard();    /* Draw screen */
        }
}


right(ptr) char *ptr;
{
        char *locn;

        while(1) {
                locn      = *(ptr) + board;
                if (standardmiddle(*(locn + 1))) return;
                *(locn+1) = *locn;
                *locn     = 0;
                (*ptr)++;
                drawboard();
        }
}

down(ptr) char *ptr;
{
        char *locn;

        while(1) {
                locn       = *(ptr) + board;
                if (standardmiddle(*(locn + 16))) return;
                *(locn+16) = *locn;
                *locn      = 0;
                (*ptr)    += 16;
                drawboard();
        }
}

up(ptr) char *ptr;
{
        char *locn;

        while(1) {
                locn       = *(ptr) + board;
                if ( standardmiddle(*(locn - 16)) ) return;
                *(locn-16) = *locn;
                *locn      = 0;
                (*ptr)    -= 16;
                drawboard();
        }
}


/* Check to see if we're running into anything, if box is set then
 * if we hit anything we want to stop, if we're ball then if we
 * hit anything except for bubble we wanna stop
 */

standardmiddle(nextpos) char nextpos;
{
        if (ballorbox)
                return(nextpos);       /* For box */
        else
                if (nextpos == BUBB || nextpos == NO) return(0);
        return(1);
}


/* Check to see if a level is finished
 * There are 144 squares in each level, note the use of != instead of
 * </<= - this is quicker to execute on the Z80!
 */

checkfinish()
{
        char *ptr;
        int   i;
        ptr = board;
        for(i = 1; i != 144; i++) {
                if (*ptr++ == BUBB) return(1);
        }
        if(++level == MAXLEVEL ) return(0);     /* Done all the levels!! */
        setuplevel();
        return(1);
}

/* Setup a level..the level is stored compressed, taking up 38 bytes a
 * time.
 *      byte 0 - position of ball
 *      byte 1 - position of box
 *      2-37   - level data
 *
 * Level data is stored as two bits per square, so we have to shift our
 * picked up byte round to get it
 */

setuplevel()
{
        int   y, x;
        char *ptr, *ptr2;
/* ptr points to start of level now */
/* First two bytes are the ball and the boxes position */
        ptr2       =  board;
        ptr        =  levels + (level * 38);
        balloffset = *ptr++;
        boxoffset  = *ptr++;

/* Now, decompress level into board */
        for (y=0; y!=9; y++) {
                for (x = 0; x != 4; x++) {
                        *ptr2++ = ((*ptr) >> 6)&3;
                        *ptr2++ = ((*ptr) >> 4)&3;
                        *ptr2++ = ((*ptr) >> 2)&3;
                        *ptr2++ = (*ptr)&3;
                         ptr++;
                }
        }
/* Now, plot the ball and box into the internal map */
        ptr2=board;
        *(ptr2+balloffset)=BALL;
        *(ptr2+boxoffset)=BOX;
        drawboard();
}

rdrwscr()
{
puts("DStar Z80\n\
Original game\n\
By A Von Dollen\n\n\
 Converted to ZX\
 By D Morris\n\
 and back by\n\
 Henk Poley");
        getkey();
}

/* Draw the board, mostly written in C, even though we did take a bit
 * of a performance hit when it was converted over from asm
 */

drawboard()
{
        int x,y;
        char *ptr;

        ptr=board;

        for (y=0;y!=9;y++) {
                for (x=0;x!=16;x++) {
                        puttiblock((*ptr++),x,y);
                }
        }
        RedrawScreen;
}


/* Dump a sprite onto the TI screen we've built
 * The TI screen is 16 characters wide by 8 deep i.e. half the size
 * of the Z88's map screen. It's stored line by line (sensible!)
 *
 * We enter with y being y/7 and x being x/8 (if we think in pixels)
 * So for each y we have to step down by 112.
 * The increment between rows is 16.
 */
 


puttiblock(spr,x,y) char spr; int x, y;
{
        char *ptr2,*ptr;
        int i;

/* We use this method instead of y*=112 because the compiler has special
 * cases for multiplying by ints less than 16 (except for 11 and 13 
 * (Hmmm, I wonder why?!?!)
 */
        y=(y*14)*8;
/* So, get where we want to dump our sprite into ptr */
        ptr=tiscr+y+x;
/* Calculate where the sprite is */
        spr*=8;
        ptr2=sprites+spr;
/* And dump it in there */
        for (i=0; i!=7;i++) {
                *ptr=*(ptr2++);
                ptr+=16;
        }
}

/* THE END! */

/* 
 * Sprite and level data for DStar
 * Stored in #asm statements
 *
 * Even though the sprites are only 7 bytes long, we align them at 8
 * bytes, so calculations go that little bit quicker!
 */

#asm
sprites:
                        ;1=edge, 2=clear ball 3=moveable ball 4=moveable block
 .db    00000000b
 .db    00000000b
 .db    00000000b
 .db    00000000b
 .db    00000000b
 .db    00000000b
 .db    00000000b
 .db    0

 .db    01111110b
 .db    10101001b
 .db    11000111b
 .db    10110001b
 .db    11001011b
 .db    10100101b
 .db    01111110b
 .db    0


 .db    00000000b
 .db    00011000b
 .db    00100100b
 .db    00100100b
 .db    00011000b
 .db    00000000b
 .db    00000000b
 .db    0

 .db    00000000b
 .db    00011000b
 .db    00110100b
 .db    00111100b
 .db    00011000b
 .db    00000000b
 .db    00000000b
 .db    0

 .db    00000000b
 .db    00111100b
 .db    00111100b
 .db    00111100b
 .db    00111100b
 .db    00000000b
 .db    00000000b
 .db    0

levels:
 .db    17,30                           ;ball offset, box offset
 .db    01010101b,01010101b,01010101b,01010101b
 .db    01000101b,00000000b,00000000b,10010001b
 .db    01000000b,00000000b,00000010b,00010101b
 .db    01000000b,00000000b,01011000b,00000001b
 .db    01000000b,01010010b,00000000b,00000101b
 .db    01010010b,00001000b,00000000b,10000001b
 .db    01001000b,00000000b,00100101b,00100001b
 .db    01000000b,00000101b,10000000b,00001001b
 .db    01010101b,01010101b,01010101b,01010101b
level2:
 .db    30,86
 .db    00010000,01000100,01000000,01000101
 .db    01000000,10000000,00000000,00000001
 .db    00000001,10000001,10000000,10000000
 .db    01000100,10000000,00001000,00010001
 .db    00000000,00000100,00001000,00000100
 .db    01000000,00010001,00001000,00000001
 .db    00000001,00000100,01000000,01101001
 .db    01000000,00000000,00000000,00000100
 .db    00010000,01000000,00000000,00010000
level3:
 .db    30,46
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000000,00000000,10010001
 .db    01000000,01010000,00000000,01010001
 .db    01000000,01100000,00000010,00000001
 .db    01001000,00000000,10010100,00001001
 .db    01000110,00001000,00100100,00100101
 .db    01000101,10000110,00001000,10010101
 .db    01100000,00000101,10000000,00000001
 .db    01010101,01010101,01010101,01010101
level4:
 .db    125,30
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000000,00000000,00010001
 .db    01000000,00000100,00000000,00000001
 .db    01011001,10001001,10011001,10011001
 .db    01000100,01100010,01000100,01000101
 .db    01011001,10011000,10011001,10011001
 .db    01000000,00000100,00000000,00000001
 .db    01000000,01000000,00000000,01000001
 .db    01010101,01010101,01010101,01010101
level5:
 .db    17,110
 .db    00010101,01010101,01010101,01010100
 .db    01000000,01000000,01000001,00000001
 .db    01000001,10000100,10000010,00010001
 .db    01010000,00000000,01000001,00000001
 .db    01100001,10010000,00000000,00000101
 .db    01010000,00000001,00100001,00000001
 .db    01100100,00010001,00010000,00010001
 .db    01000000,01000000,00100100,00011001
 .db    00010101,01010101,01010101,01010100
level6:
 .db    65,113
 .db    00000000,01010101,01010101,01010101
 .db    00000001,00000010,00000001,10001001
 .db    00000100,00000010,00000000,01000101
 .db    00010000,00000010,00000000,00000001
 .db    01000000,00000010,00000000,00000001
 .db    01010000,00000010,00000100,00000101
 .db    01000000,00000010,00000000,01000001
 .db    01000001,00000010,00000101,10000001
 .db    01010101,01010101,01010101,01010101
level7:
 .db    115,122
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000000,00000000,00000001
 .db    00010100,01010100,00011000,01011001
 .db    00011000,00011000,01000100,01000100
 .db    00000100,00010000,01010100,01010000
 .db    00010100,00010000,01100100,01100100
 .db    01000000,00000000,00000000,00000001
 .db    01000000,01100000,00000000,00011001
 .db    01010101,01010101,01010101,01010101
level8:
 .db    108,98
 .db    01010101,01010101,01010101,01010100
 .db    01000010,01010000,00000000,00000101
 .db    01000001,10000001,01001000,00000001
 .db    01000010,01010001,00011000,00000001
 .db    01010000,00000001,01000001,10010001
 .db    01010001,00000000,00000010,01100001
 .db    01100010,01000000,10000001,00010001
 .db    01010000,00000000,00000000,00000001
 .db    00010101,01010101,01010101,01010101
level9:
 .db    30,72
 .db    00000100,01010101,01010101,01010100
 .db    00011001,10000000,00000001,00000001
 .db    01100010,01000000,00100000,00000100
 .db    00010001,00001001,01000010,01000001
 .db    01000001,10000110,00100000,00001001
 .db    01000000,00001001,01000000,00000100
 .db    01100110,00000000,00000000,00010000
 .db    01000000,00000000,00000000,01000000
 .db    01010101,01010101,01010101,00000000
level10:
 .db    93,36
 .db    00000000,01010101,01010101,01010100
 .db    01010101,00100000,00000000,00000001
 .db    01000000,00000101,01100010,01001001
 .db    01001000,00000110,00011000,00000100
 .db    01000000,00000100,00100000,01001001
 .db    01100110,00000100,10010000,01000100
 .db    00011000,00000101,01000001,01010000
 .db    01000000,00000000,00000100,01000100
 .db    00010101,01010101,01010000,01000001
level11:
 .db    30,108
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000001,00000000,00000001
 .db    01000001,10100000,00000010,10000101
 .db    01010000,00100000,00010100,00001001
 .db    01100000,00000110,01101000,00010101
 .db    01010001,01000000,00010100,00000001
 .db    01100000,10010010,00000000,00001001
 .db    01011001,01010000,00000100,00000101
 .db    00010100,01010101,01010101,01010100
level12:
 .db    17,92
 .db    01010000,00000001,01000001,01010100
 .db    01000101,01010110,00010101,00100101
 .db    01000000,00101000,00000000,10000001
 .db    01000101,00000101,10000001,10010001
 .db    01000100,10000101,01100001,01000001
 .db    01000101,00000101,00000001,00010001
 .db    01000000,00001000,00000000,00000001
 .db    01000000,00000000,00100000,00000001
 .db    01010101,01010101,01010101,01010101
level13:
 .db    18,113
 .db    00010101,01010101,01010101,01010100
 .db    01000001,00000000,00000000,10000101
 .db    01000100,00000110,00000010,01010001
 .db    01000000,00000000,10000000,00010001
 .db    01001000,00000000,00000000,00011001
 .db    01000100,00000000,00100000,00000001
 .db    01010000,00000000,10001000,00011001
 .db    01000000,01000000,00100001,00010001
 .db    00010101,01010101,01010101,01010100
level14:
 .db    36,50
 .db    01010101,01010101,01010101,01010101
 .db    01100110,00000000,00000000,10011001
 .db    01001001,00000000,00000001,01000001
 .db    01000000,00000000,00000010,00000001
 .db    01000000,00000000,00100100,00000001
 .db    01000000,00000010,00000000,00000001
 .db    01001001,00000000,00000000,01000001
 .db    01100110,00000000,00000000,10011001
 .db    01010101,01010101,01010101,01010101
level15:
 .db    51,76
 .db    00010101,01010100,01010101,01010100
 .db    01000000,00001001,00000000,00100001
 .db    01000100,10000100,00010000,00100001
 .db    01000000,01000000,01101000,01100001
 .db    00010001,00000001,00100000,00010001
 .db    01100000,00000000,00010000,01100001
 .db    00010000,00000000,10000000,00000100
 .db    01100000,00000000,00000000,00001001
 .db    00010101,01010101,01010101,01010100
level16:
 .db    35,19
 .db    01010101,01010101,01010101,01010101
 .db    01010000,01100010,00000000,00001001
 .db    01100000,10011000,00000000,00000101
 .db    01010001,01010000,00001000,00000101
 .db    01010000,00000010,01100100,00000001
 .db    01101000,00000000,00001001,10000001
 .db    01010010,00000000,01010101,10000001
 .db    01011001,00000100,00000000,00000001
 .db    01010101,01010101,01010101,01010101
level17:
 .db    29,124
 .db    01010101,01010101,01010101,01010101
 .db    01001001,00000000,00000000,01000001
 .db    01000100,00100110,10011000,00010001
 .db    01000000,00011001,01100100,10000001
 .db    01001001,00000000,00000010,01000001
 .db    01000010,01100000,00001001,00000001
 .db    01000100,00010001,01100100,00010001
 .db    01000000,00100001,10000000,00000001
 .db    01010101,01010101,01010101,01010101
level18:
 .db    115,26
 .db    01010101,01010101,01010101,01010101
 .db    01001000,00000010,00000001,00000001
 .db    01000001,10011000,00000110,00000001
 .db    01000000,01100100,00000001,10000001
 .db    01000000,10000001,00000010,01100001
 .db    01000110,01000000,01001001,00000001
 .db    01001001,10000100,10000100,00000001
 .db    01100100,00000100,00000000,01000001
 .db    01010101,01010101,01010101,01010101
level19:
 .db    126,110
 .db    01010101,01010101,01010101,01010101
 .db    01100000,00010100,00000000,01011001
 .db    01000100,00010000,00000000,01100001
 .db    01001001,00000010,01010000,10000001
 .db    01000100,00000001,10000000,00000001
 .db    01000000,00010000,00100100,00000001
 .db    01000101,00100100,01011000,00010001
 .db    01001001,00011000,00000000,01010001
 .db    01010101,01010101,01010101,01010101
level20:
 .db    77,66
 .db    01010101,01010101,01010101,01010101
 .db    01000000,10011000,00000000,00000001
 .db    01011000,00100100,01011000,00000101
 .db    01000100,01001000,00000100,00010001
 .db    01000000,01000001,01000001,00001001
 .db    01000100,00010000,00100001,00010001
 .db    01010000,00100101,00011000,00100101
 .db    01000000,00000000,00100110,00000001
 .db    01010101,01010101,01010101,01010101
level21:
 .db    103,105
 .db    01010101,01010101,01010101,01010101
 .db    01000101,01000000,00001000,00000101
 .db    01000000,01000000,00000000,01000101
 .db    01000000,01011000,00000000,00100001
 .db    01000010,00000000,10000000,10000101
 .db    01000000,00010000,00000101,01100001
 .db    01000010,00100000,00000010,00101001
 .db    01010101,01010101,01010101,01010101
 .db    00000000,00000000,00000000,00000000
level22:
 .db    103,105
 .db    01010101,01010101,01010101,01010101
 .db    01100100,00011001,00011000,00010001
 .db    01000000,00010000,00000000,00000001
 .db    01100000,00010000,01100000,10000001
 .db    01010001,10000000,00000010,00010101
 .db    01001000,01000000,01010110,00000001
 .db    01000000,00000100,01000000,10000001
 .db    01010101,01010101,01010101,01010101
 .db    00000000,00000000,00000000,00000000
level23:
 .db    103,105
 .db    00010101,01010101,01010101,01010100
 .db    01000100,00011001,00011000,00010001
 .db    01000000,00100000,01000000,00000001
 .db    01010000,00010000,00100001,10000001
 .db    01000001,10000001,00001010,00100001
 .db    01011000,01000000,01010010,00000001
 .db    01000000,00000100,01000000,10000001
 .db    00010101,01010101,01010101,01010100
 .db    00000000,00000000,00000000,00000000
level24:
 .db    103,105
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000100,00000000,00000101
 .db    01000101,10001000,00000001,01100101
 .db    01000110,00000000,00100100,00010101
 .db    01001010,00001001,00010100,00000001
 .db    01000110,00100001,00000000,01010001
 .db    01000101,00000000,01000101,01101001
 .db    01010101,01010101,01010101,01010101
 .db    00000000,00000000,00000000,00000000
level25:
 .db    103,105
 .db    01010101,01010101,01010101,01010101
 .db    01000000,00000000,00000000,00010001
 .db    01001000,01011000,00001000,00000001
 .db    01000000,01100000,10000001,01000001
 .db    01001000,00000001,01000001,10000001
 .db    01000110,00000010,01000000,00100001
 .db    01000101,10000100,00000000,00000001
 .db    01010101,01010101,01010101,01010101
 .db    00000000,00000000,00000000,00000000

#endasm

